@@ -142,6 +142,8 @@ write_tunnel_params(GString* s, const NetplanNetDefinition* def)
         g_string_append_printf(params, "Mode=%s\n", tunnel_mode_to_string(def->tunnel.mode));
     g_string_append_printf(params, "Local=%s\n", def->tunnel.local_ip);
     g_string_append_printf(params, "Remote=%s\n", def->tunnel.remote_ip);
+    if (def->tunnel_ttl)
+        g_string_append_printf(params, "TTL=%u\n", def->tunnel_ttl);
     if (def->tunnel.input_key)
         g_string_append_printf(params, "InputKey=%s\n", def->tunnel.input_key);
     if (def->tunnel.output_key)
@@ -227,7 +229,7 @@ write_link_file(const NetplanNetDefinition* def, const char* rootdir, const char
         return;
 
     /* do we need to write a .link file? */
-    if (!def->set_name && !def->wake_on_lan && !def->mtubytes && !def->set_mac)
+    if (!def->set_name && !def->wake_on_lan && !def->mtubytes)
         return;
 
     /* build file contents */
@@ -241,9 +243,6 @@ write_link_file(const NetplanNetDefinition* def, const char* rootdir, const char
     g_string_append_printf(s, "WakeOnLan=%s\n", def->wake_on_lan ? "magic" : "off");
     if (def->mtubytes)
         g_string_append_printf(s, "MTUBytes=%u\n", def->mtubytes);
-    if (def->set_mac)
-        g_string_append_printf(s, "MACAddress=%s\n", def->set_mac);
-
 
     orig_umask = umask(022);
     g_string_free_to_file(s, rootdir, path, ".link");
@@ -423,9 +422,14 @@ write_netdev_file(const NetplanNetDefinition* def, const char* rootdir, const ch
 static void
 write_route(NetplanIPRoute* r, GString* s)
 {
+    const char *to;
     g_string_append_printf(s, "\n[Route]\n");
 
-    g_string_append_printf(s, "Destination=%s\n", r->to);
+    if (g_strcmp0(r->to, "default") == 0)
+        to = get_global_network(r->family);
+    else
+        to = r->to;
+    g_string_append_printf(s, "Destination=%s\n", to);
 
     if (r->via)
         g_string_append_printf(s, "Gateway=%s\n", r->via);
@@ -444,6 +448,10 @@ write_route(NetplanIPRoute* r, GString* s)
         g_string_append_printf(s, "Table=%d\n", r->table);
     if (r->mtubytes != NETPLAN_MTU_UNSPEC)
         g_string_append_printf(s, "MTUBytes=%u\n", r->mtubytes);
+    if (r->congestion_window != NETPLAN_CONGESTION_WINDOW_UNSPEC)
+        g_string_append_printf(s, "InitialCongestionWindow=%u\n", r->congestion_window);
+    if (r->advertised_receive_window != NETPLAN_ADVERTISED_RECEIVE_WINDOW_UNSPEC)
+        g_string_append_printf(s, "InitialAdvertisedReceiveWindow=%u\n", r->advertised_receive_window);
 }
 
 static void
@@ -546,6 +554,7 @@ write_network_file(const NetplanNetDefinition* def, const char* rootdir, const c
     GString* link = NULL;
     GString* s = NULL;
     mode_t orig_umask;
+    gboolean is_optional = def->optional;
 
     if (def->type == NETPLAN_DEF_TYPE_VLAN && def->sriov_vlan_filter) {
         g_debug("%s is defined as a hardware SR-IOV filtered VLAN, postponing creation", def->id);
@@ -558,8 +567,23 @@ write_network_file(const NetplanNetDefinition* def, const char* rootdir, const c
     /* Prepare the [Network] section */
     network = g_string_sized_new(200);
 
-    if (def->optional || def->optional_addresses) {
-        if (def->optional) {
+    /* The ActivationPolicy setting is available in systemd v248+ */
+    if (def->activation_mode) {
+        const char* mode;
+        if (g_strcmp0(def->activation_mode, "manual") == 0)
+            mode = "manual";
+        else /* "off" */
+            mode = "always-down";
+        g_string_append_printf(link, "ActivationPolicy=%s\n", mode);
+        /* When activation-mode is used we default to being optional.
+         * Otherwise systemd might wait indefinitely for the interface to
+         * become online.
+         */
+        is_optional = TRUE;
+    }
+
+    if (is_optional || def->optional_addresses) {
+        if (is_optional) {
             g_string_append(link, "RequiredForOnline=no\n");
         }
         for (unsigned i = 0; NETPLAN_OPTIONAL_ADDRESS_TYPES[i].name != NULL; ++i) {
@@ -569,13 +593,13 @@ write_network_file(const NetplanNetDefinition* def, const char* rootdir, const c
         }
     }
 
-    if (def->mtubytes) {
+    if (def->mtubytes)
         g_string_append_printf(link, "MTUBytes=%u\n", def->mtubytes);
-    }
+    if (def->set_mac)
+        g_string_append_printf(link, "MACAddress=%s\n", def->set_mac);
 
-    if (def->emit_lldp) {
+    if (def->emit_lldp)
         g_string_append(network, "EmitLLDP=true\n");
-    }
 
     if (def->dhcp4 && def->dhcp6)
         g_string_append(network, "DHCP=yes\n");
@@ -822,6 +846,8 @@ append_wpa_auth_conf(GString* s, const NetplanAuthenticationSettings* auth, cons
         case NETPLAN_AUTH_KEY_MANAGEMENT_8021X:
             g_string_append(s, "  key_mgmt=IEEE8021X\n");
             break;
+
+        default: break; // LCOV_EXCL_LINE
     }
 
     switch (auth->eap_method) {
@@ -839,6 +865,8 @@ append_wpa_auth_conf(GString* s, const NetplanAuthenticationSettings* auth, cons
         case NETPLAN_AUTH_EAP_TTLS:
             g_string_append(s, "  eap=TTLS\n");
             break;
+
+        default: break; // LCOV_EXCL_LINE
     }
 
     if (auth->identity) {
@@ -897,7 +925,6 @@ append_wpa_auth_conf(GString* s, const NetplanAuthenticationSettings* auth, cons
 static void
 write_wpa_unit(const NetplanNetDefinition* def, const char* rootdir)
 {
-    g_autoptr(GError) err = NULL;
     g_autofree gchar *stdouth = NULL;
 
     stdouth = systemd_escape(def->id);
@@ -974,8 +1001,8 @@ write_wpa_conf(const NetplanNetDefinition* def, const char* rootdir)
                 case NETPLAN_WIFI_MODE_ADHOC:
                     g_string_append(s, "  mode=1\n");
                     break;
-                case NETPLAN_WIFI_MODE_AP:
-                    g_fprintf(stderr, "ERROR: %s: networkd does not support wifi in access point mode\n", def->id);
+                default:
+                    g_fprintf(stderr, "ERROR: %s: %s: networkd does not support this wifi mode\n", def->id, ap->ssid);
                     exit(1);
             }
 
