@@ -205,6 +205,8 @@ validate_tunnel_grammar(NetplanNetDefinition* nd, yaml_node_t* node, GError** er
         return yaml_error(node, error, "%s: missing 'local' property for tunnel", nd->id);
     if (!nd->tunnel.remote_ip)
         return yaml_error(node, error, "%s: missing 'remote' property for tunnel", nd->id);
+    if (nd->tunnel_ttl && nd->tunnel_ttl > 255)
+        return yaml_error(node, error, "%s: 'ttl' property for tunnel must be in range [1...255]", nd->id);
 
     switch(nd->tunnel.mode) {
         case NETPLAN_TUNNEL_MODE_IPIP6:
@@ -340,6 +342,9 @@ validate_netdef_grammar(NetplanNetDefinition* nd, yaml_node_t* node, GError** er
         // LCOV_EXCL_STOP
     }
 
+    if (nd->type == NETPLAN_DEF_TYPE_NM && (!nd->backend_settings.nm.passthrough || !g_datalist_get_data(&nd->backend_settings.nm.passthrough, "connection.type")))
+        return yaml_error(node, error, "%s: network type 'nm-devices:' needs to provide a 'connection.type' via passthrough", nd->id);
+
     valid = TRUE;
 
 netdef_grammar_error:
@@ -367,3 +372,115 @@ backend_rules_error:
     return valid;
 }
 
+struct _defroute_entry {
+    int family;
+    int table;
+    int metric;
+    const char *netdef_id;
+};
+
+static void
+defroute_err(struct _defroute_entry *entry, const char *new_netdef_id, GError **error) {
+    char table_name[128] = {};
+    char metric_name[128] = {};
+
+    g_assert(entry->family == AF_INET || entry->family == AF_INET6);
+
+    // XXX: handle 254 as an alias for main ?
+    if (entry->table == NETPLAN_ROUTE_TABLE_UNSPEC)
+        strncpy(table_name, "table: main", sizeof(table_name) - 1);
+    else
+        snprintf(table_name, sizeof(table_name) - 1, "table: %d", entry->table);
+
+    if (entry->metric == NETPLAN_METRIC_UNSPEC)
+        strncpy(metric_name, "metric: default", sizeof(metric_name) - 1);
+    else
+        snprintf(metric_name, sizeof(metric_name) - 1, "metric: %d", entry->metric);
+
+    g_set_error(error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+            "Conflicting default route declarations for %s (%s, %s), first declared in %s but also in %s",
+            (entry->family == AF_INET) ? "IPv4" : "IPv6",
+            table_name,
+            metric_name,
+            entry->netdef_id,
+            new_netdef_id);
+}
+
+static gboolean
+check_defroute(struct _defroute_entry *candidate,
+               GSList **entries,
+               GError **error)
+{
+    struct _defroute_entry *entry;
+    GSList *it;
+
+    g_assert(entries != NULL);
+    it = *entries;
+
+    while (it) {
+        struct _defroute_entry *e = it->data;
+        if (e->family == candidate->family &&
+                e->table == candidate->table &&
+                e->metric == candidate->metric) {
+            defroute_err(e, candidate->netdef_id, error);
+            return FALSE;
+        }
+        it = it->next;
+    }
+    entry = g_malloc(sizeof(*entry));
+    *entry = *candidate;
+    *entries = g_slist_prepend(*entries, entry);
+    return TRUE;
+}
+
+gboolean
+validate_default_route_consistency(GHashTable *netdefs, GError ** error)
+{
+    struct _defroute_entry candidate = {};
+    GSList *defroutes = NULL;
+    gboolean ret = TRUE;
+    gpointer key, value;
+    GHashTableIter iter;
+
+    g_hash_table_iter_init (&iter, netdefs);
+    while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+        NetplanNetDefinition *nd = value;
+        candidate.netdef_id = key;
+        candidate.metric = NETPLAN_METRIC_UNSPEC;
+        candidate.table = NETPLAN_ROUTE_TABLE_UNSPEC;
+        if (nd->gateway4) {
+            candidate.family = AF_INET;
+            if (!check_defroute(&candidate, &defroutes, error)) {
+                ret = FALSE;
+                break;
+            }
+        }
+        if (nd->gateway6) {
+            candidate.family = AF_INET6;
+            if (!check_defroute(&candidate, &defroutes, error)) {
+                ret = FALSE;
+                break;
+            }
+        }
+
+        if (!nd->routes)
+            continue;
+
+        for (size_t i = 0; i < nd->routes->len; i++) {
+            NetplanIPRoute* r = g_array_index(nd->routes, NetplanIPRoute*, i);
+            char *suffix = strrchr(r->to, '/');
+            if (g_strcmp0(suffix, "/0") == 0 || g_strcmp0(r->to, "default") == 0) {
+                candidate.family = r->family;
+                candidate.table = r->table;
+                candidate.metric = r->metric;
+                if (!check_defroute(&candidate, &defroutes, error)) {
+                    ret = FALSE;
+                    break;
+                }
+            }
+        }
+    }
+    g_slist_free_full(defroutes, g_free);
+    return ret;
+}
