@@ -132,6 +132,7 @@ class TestNetplanDBus(unittest.TestCase):
             exe_cli + ["apply"],
             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
             env=newenv)
+        p.wait(10)
         self.assertEqual(p.stdout.read(), b"")
         self.assertEqual(p.stderr.read(), b"")
         self.assertEquals(self.mock_busctl_cmd.calls(), [
@@ -143,6 +144,76 @@ class TestNetplanDBus(unittest.TestCase):
              ],
         ])
 
+    def test_netplan_apply_in_snap_calls_busctl_ret130(self):
+        newenv = os.environ.copy()
+        busctlDir = os.path.dirname(self.mock_busctl_cmd.path)
+        newenv["PATH"] = busctlDir+":"+os.environ["PATH"]
+        self.mock_busctl_cmd.set_returncode(130)
+        p = subprocess.Popen(
+            exe_cli + ["apply"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            env=newenv)
+        p.wait(10)
+        # exit_on_error is True by default, so we check the returncode directly
+        self.assertEqual(p.returncode, 130)
+
+    def test_netplan_apply_in_snap_calls_busctl_err(self):
+        newenv = os.environ.copy()
+        busctlDir = os.path.dirname(self.mock_busctl_cmd.path)
+        newenv["PATH"] = busctlDir+":"+os.environ["PATH"]
+        self.mock_busctl_cmd.set_returncode(1)
+        p = subprocess.Popen(
+            exe_cli + ["apply"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            env=newenv)
+        p.wait(10)
+        # exit_on_error is True by default, so we check the returncode directly
+        self.assertEqual(p.returncode, 1)
+
+    def test_netplan_generate_in_snap_calls_busctl(self):
+        newenv = os.environ.copy()
+        busctlDir = os.path.dirname(self.mock_busctl_cmd.path)
+        newenv["PATH"] = busctlDir+":"+os.environ["PATH"]
+        p = subprocess.Popen(
+            exe_cli + ["generate"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            env=newenv)
+        p.wait(10)
+        self.assertEqual(p.stdout.read(), b"")
+        self.assertEqual(p.stderr.read(), b"")
+        self.assertEquals(self.mock_busctl_cmd.calls(), [
+            ["busctl", "call", "--quiet", "--system",
+             "io.netplan.Netplan",  # the service
+             "/io/netplan/Netplan",  # the object
+             "io.netplan.Netplan",  # the interface
+             "Generate",  # the method
+             ],
+        ])
+
+    def test_netplan_generate_in_snap_calls_busctl_ret130(self):
+        newenv = os.environ.copy()
+        busctlDir = os.path.dirname(self.mock_busctl_cmd.path)
+        newenv["PATH"] = busctlDir+":"+os.environ["PATH"]
+        self.mock_busctl_cmd.set_returncode(130)
+        p = subprocess.Popen(
+            exe_cli + ["generate"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            env=newenv)
+        p.wait(10)
+        self.assertIn(b"PermissionError: failed to communicate with dbus service", p.stderr.read())
+
+    def test_netplan_generate_in_snap_calls_busctl_err(self):
+        newenv = os.environ.copy()
+        busctlDir = os.path.dirname(self.mock_busctl_cmd.path)
+        newenv["PATH"] = busctlDir+":"+os.environ["PATH"]
+        self.mock_busctl_cmd.set_returncode(1)
+        p = subprocess.Popen(
+            exe_cli + ["generate"],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            env=newenv)
+        p.wait(10)
+        self.assertIn(b"RuntimeError: failed to communicate with dbus service: error 1", p.stderr.read())
+
     def test_netplan_dbus_noroot(self):
         # Process should fail instantly, if not: kill it after 5 sec
         r = subprocess.run(NETPLAN_DBUS_CMD, timeout=5, capture_output=True)
@@ -173,6 +244,21 @@ class TestNetplanDBus(unittest.TestCase):
                 ["netplan", "apply"],
         ])
 
+    def test_netplan_dbus_generate(self):
+        BUSCTL_NETPLAN_CMD = [
+            "busctl", "call", "--system",
+            "io.netplan.Netplan",
+            "/io/netplan/Netplan",
+            "io.netplan.Netplan",
+            "Generate",
+        ]
+        output = subprocess.check_output(BUSCTL_NETPLAN_CMD)
+        self.assertEqual(output.decode("utf-8"), "b true\n")
+        # one call to netplan apply in total
+        self.assertEquals(self.mock_netplan_cmd.calls(), [
+                ["netplan", "generate"],
+        ])
+
     def test_netplan_dbus_info(self):
         BUSCTL_NETPLAN_INFO = [
             "busctl", "call", "--system",
