@@ -435,6 +435,122 @@ class TestConfigErrors(TestBase):
       gateway6: %s''' % a, expect_fail=True)
             self.assertIn("invalid IPv6 address '%s'" % a, err)
 
+    def test_multiple_ip4_gateways(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: [192.168.22.78/24]
+      gateway4: 192.168.22.1
+    enblue:
+      addresses: [10.49.34.4/16]
+      gateway4: 10.49.2.38''', expect_fail=False)
+        self.assertIn("Problem encountered while validating default route consistency", err)
+        self.assertIn("Conflicting default route declarations for IPv4 (table: main, metric: default)", err)
+        self.assertIn("engreen", err)
+        self.assertIn("enblue", err)
+
+    def test_multiple_ip6_gateways(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: [2001:FFfe::1/62]
+      gateway6: 2001:FFfe::2
+    enblue:
+      addresses: [2001:FFfe::33/62]
+      gateway6: 2001:FFfe::34''', expect_fail=False)
+        self.assertIn("Problem encountered while validating default route consistency", err)
+        self.assertIn("Conflicting default route declarations for IPv6 (table: main, metric: default)", err)
+        self.assertIn("engreen", err)
+        self.assertIn("enblue", err)
+
+    def test_gateway_and_default_route(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: [10.49.34.4/16]
+      gateway4: 10.49.2.38
+      routes:
+      - to: default
+        via: 10.49.65.89''', expect_fail=False)
+        self.assertIn("Problem encountered while validating default route consistency", err)
+        self.assertIn("Conflicting default route declarations for IPv4 (table: main, metric: default)", err)
+        self.assertIn("engreen", err)
+
+    def test_multiple_default_routes_on_other_table(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: [10.49.34.4/16]
+      routes:
+      - to: default
+        via: 10.49.65.89
+    enblue:
+      addresses: [10.50.35.3/16]
+      routes:
+      - to: default
+        via: 10.49.65.89
+        table: 23
+    enred:
+      addresses: [172.137.1.4/24]
+      routes:
+      - to: default
+        via: 172.137.1.1
+        table: 23
+        ''', expect_fail=False)
+        self.assertIn("Problem encountered while validating default route consistency", err)
+        self.assertIn("Conflicting default route declarations for IPv4 (table: 23, metric: default)", err)
+        self.assertIn("enblue", err)
+        self.assertIn("enred", err)
+        self.assertNotIn("engreen", err)
+
+    def test_multiple_default_routes_on_specific_metrics(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: [10.49.34.4/16]
+      routes:
+      - to: default
+        via: 10.49.65.89
+        metric: 100
+    enblue:
+      addresses: [10.50.35.3/16]
+      routes:
+      - to: default
+        via: 10.49.65.89
+        metric: 600
+    enred:
+      addresses: [172.137.1.4/24]
+      routes:
+      - to: default
+        via: 172.137.1.1
+        metric: 600
+        ''', expect_fail=False)
+        self.assertIn("Problem encountered while validating default route consistency", err)
+        self.assertIn("Conflicting default route declarations for IPv4 (table: main, metric: 600)", err)
+        self.assertIn("enblue", err)
+        self.assertIn("enred", err)
+        self.assertNotIn("engreen", err)
+
+    def test_default_route_and_0(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      addresses: [10.49.34.4/16]
+      routes:
+      - to: default
+        via: 10.49.65.89
+      - to: 0.0.0.0/0
+        via: 10.49.65.67''', expect_fail=False)
+        self.assertIn("Problem encountered while validating default route consistency", err)
+        self.assertIn("Conflicting default route declarations for IPv4 (table: main, metric: default)", err)
+        self.assertIn("engreen", err)
+
     def test_invalid_nameserver_ipv4(self):
         for a in ['300.400.1.1', '1.2.3', '192.168.14.1/24']:
             err = self.generate('''network:
@@ -850,3 +966,11 @@ class TestConfigErrors(TestBase):
     engreen:
       dhcp4: *yes''', expect_fail=True)
         self.assertIn("aliases are not supported", err)
+
+    def test_invalid_activation_mode(self):
+        err = self.generate('''network:
+  version: 2
+  ethernets:
+    engreen:
+      activation-mode: invalid''', expect_fail=True)
+        self.assertIn("needs to be 'manual' or 'off'", err)
