@@ -5,8 +5,8 @@
 # These need to be run in a VM and do change the system
 # configuration.
 #
-# Copyright (C) 2020 Canonical, Ltd.
-# Author: Lukas Märdian <lukas.maerdian@canonical.com>
+# Copyright (C) 2020-2021 Canonical, Ltd.
+# Author: Lukas Märdian <slyon@ubuntu.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -73,18 +73,13 @@ class _CommonTests():
         self.addCleanup(subprocess.call, ['ovs-vsctl', '--if-exists', 'del-port', 'patch1-0'])
         with open(self.config, 'w') as f:
             f.write('''network:
-  ethernets:
-    # Add a normal interface, to avoid networkd-wait-online.service timeout.
-    # If we have just OVS interfaces/ports networkd/networkctl will not be
-    # aware that our network is ready.
-    %(ec)s: {addresses: [10.10.10.20/24]}
   openvswitch:
     ports:
       - [patch0-1, patch1-0]
   bridges:
     ovs0: {interfaces: [patch0-1]}
-    ovs1: {interfaces: [patch1-0]}''' % {'ec': self.dev_e_client})
-        self.generate_and_settle()
+    ovs1: {interfaces: [patch1-0]}''')
+        self.generate_and_settle(['ovs0', 'ovs1'])
         # Basic verification that the bridges/ports/interfaces are there in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'])
         self.assertIn(b'    Bridge ovs0', out)
@@ -97,7 +92,7 @@ class _CommonTests():
             f.write('''network:
   ethernets:
     %(ec)s: {addresses: ['1.2.3.4/24']}''' % {'ec': self.dev_e_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client])
         # Verify that the netplan=true tagged bridges/ports have been cleaned up
         out = subprocess.check_output(['ovs-vsctl', 'show'])
         self.assertNotIn(b'Bridge ovs0', out)
@@ -125,7 +120,7 @@ class _CommonTests():
     bond0: {interfaces: [patch1-0, %(ec)s]}
   bridges:
     ovs0: {interfaces: [patch0-1, bond0]}''' % {'ec': self.dev_e_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client, 'ovs0'])
         # Basic verification that the bridges/ports/interfaces are there in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'])
         self.assertIn(b'    Bridge ovs0', out)
@@ -143,7 +138,7 @@ class _CommonTests():
     bond0: {interfaces: [patchx, %(ec)s]}
   bridges:
     ovs1: {interfaces: [patchy, bond0]}''' % {'ec': self.dev_e_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client, 'ovs1'])
         # Verify that the netplan=true tagged patch ports have been cleaned up
         # even though the containing bond0 port still exists (with new patch ports)
         out = subprocess.check_output(['ovs-vsctl', 'show'])
@@ -183,7 +178,10 @@ class _CommonTests():
         br-%(ec)s.100:
             id: 100
             link: br-%(ec)s''' % {'ec': self.dev_e_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client,
+                                  self.state_dhcp4('br-eth42'),
+                                  'br-data',
+                                  'br-eth42.100'])
         # Basic verification that the interfaces/ports are set up in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'])
         self.assertIn(b'    Bridge br-%b' % self.dev_e_client.encode(), out)
@@ -227,7 +225,7 @@ class _CommonTests():
         controller:
           addresses: [tcp:127.0.0.1, "pssl:1337:[::1]", unix:/some/socket]
 ''' % {'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client, self.dev_e2_client, 'ovsbr'])
         # Basic verification that the interfaces/ports are in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'])
         self.assertIn(b'    Bridge ovsbr', out)
@@ -263,7 +261,7 @@ class _CommonTests():
     ovsbr:
       addresses: [192.170.1.1/24]
       interfaces: [mybond]''' % {'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client, self.dev_e2_client, 'ovsbr'])
         # Basic verification that the interfaces/ports are in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'])
         self.assertIn(b'    Bridge ovsbr', out)
@@ -278,12 +276,12 @@ class _CommonTests():
         self.assertIn(b'---- mybond ----', out)
         self.assertIn(b'bond_mode: balance-slb', out)
         self.assertIn(b'lacp_status: off', out)
-        self.assertIn(b'slave %b: enabled' % self.dev_e_client.encode(), out)
-        self.assertIn(b'slave %b: enabled' % self.dev_e2_client.encode(), out)
+        self.assertRegex(out, br'(slave|member) %b: enabled' % self.dev_e_client.encode())
+        self.assertRegex(out, br'(slave|member) %b: enabled' % self.dev_e2_client.encode())
         self.assert_iface('ovsbr', ['inet 192.170.1.1/24'])
 
     def test_bridge_patch_ports(self):
-        self.setup_eth(None, False)
+        self.setup_eth(None)
         self.addCleanup(subprocess.call, ['ovs-vsctl', '--if-exists', 'del-br', 'br0'])
         self.addCleanup(subprocess.call, ['ovs-vsctl', '--if-exists', 'del-br', 'br1'])
         self.addCleanup(subprocess.call, ['ovs-vsctl', '--if-exists', 'del-port', 'patch0-1'])
@@ -300,7 +298,7 @@ class _CommonTests():
     br1:
       addresses: [192.168.2.1/24]
       interfaces: [patch1-0]''')
-        self.generate_and_settle()
+        self.generate_and_settle(['br0', 'br1'])
         # Basic verification that the interfaces/ports are set up in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'])
         self.assertIn(b'    Bridge br0', out)
@@ -333,7 +331,7 @@ class _CommonTests():
         ovs-br:
             interfaces: [non-ovs-bond]
             openvswitch: {}''' % {'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client, self.dev_e2_client, 'ovs-br', 'non-ovs-bond'])
         # Basic verification that the interfaces/ports are set up in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'], universal_newlines=True)
         self.assertIn('    Bridge ovs-br', out)
@@ -379,7 +377,7 @@ class _CommonTests():
             id: 21
             link: %(ec)s
             mtu: 1500''' % {'ec': self.dev_e_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client, 'ovs0', 'eth42.21'])
         # Basic verification that the interfaces/ports are set up in OVS
         out = subprocess.check_output(['ovs-vsctl', 'show'], universal_newlines=True)
         self.assertIn('    Bridge ovs0', out)
@@ -428,7 +426,7 @@ class _CommonTests():
       certificate: /another/cert.pem
       private-key: /private/key.pem
     external-ids:
-      somekey: somevalue
+      somekey: 55:44:33:22:11:00
     other-config:
       key: value
   ethernets:
@@ -462,13 +460,14 @@ class _CommonTests():
           iface-id: myhostname
         other-config:
           disable-in-band: true
+          hwaddr: aa:bb:cc:dd:ee:ff
     ovs1:
       openvswitch:
         # Add ovs1 as rstp cannot be used if bridge contains a bond interface
         rstp: true
 
 ''' % {'ec': self.dev_e_client, 'e2c': self.dev_e2_client})
-        self.generate_and_settle()
+        self.generate_and_settle([self.dev_e_client, self.dev_e2_client, 'ovs0', 'ovs1'])
         before = self._collect_ovs_settings('ovs0')
         subprocess.check_call(['netplan', 'apply', '--only-ovs-cleanup'])
         after = self._collect_ovs_settings('ovs0')
@@ -522,13 +521,15 @@ class _CommonTests():
         # Verify other-config
         self.assertIn(b'key=value', before['other-config-Open_vSwitch'])
         self.assertNotIn(b'key=value', after['other-config-Open_vSwitch'])
-        self.assertIn(b'ovs0,disable-in-band=true\n', before['other-config-Bridge'])
+        self.assertIn(b'hwaddr=aa:bb:cc:dd:ee:ff', before['other-config-Bridge'])
+        self.assertNotIn(b'hwaddr=aa:bb:cc:dd:ee:ff', after['other-config-Bridge'])
+        self.assertIn(b'ovs0,disable-in-band=true', before['other-config-Bridge'])
         self.assertIn(b'ovs0,\n', after['other-config-Bridge'])
         self.assertIn(b'eth42,disable-in-band=false\n', before['other-config-Interface'])
         self.assertIn(b'eth42,\n', after['other-config-Interface'])
         # Verify external-ids
-        self.assertIn(b'somekey=somevalue', before['external-ids-Open_vSwitch'])
-        self.assertNotIn(b'somekey=somevalue', after['external-ids-Open_vSwitch'])
+        self.assertIn(b'somekey=55:44:33:22:11:00', before['external-ids-Open_vSwitch'])
+        self.assertNotIn(b'somekey=55:44:33:22:11:00', after['external-ids-Open_vSwitch'])
         self.assertIn(b'iface-id=myhostname', before['external-ids-Bridge'])
         self.assertNotIn(b'iface-id=myhostname', after['external-ids-Bridge'])
         self.assertIn(b'iface-id=mylocaliface', before['external-ids-Interface'])
